@@ -35,7 +35,8 @@ G_DEFINE_TYPE (BgPicturesSource, bg_pictures_source, BG_TYPE_SOURCE)
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), BG_TYPE_PICTURES_SOURCE, BgPicturesSourcePrivate))
 
 #define ATTRIBUTES G_FILE_ATTRIBUTE_STANDARD_NAME "," \
-	G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
+	G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," \
+        G_FILE_ATTRIBUTE_TIME_MODIFIED
 
 struct _BgPicturesSourcePrivate
 {
@@ -43,6 +44,9 @@ struct _BgPicturesSourcePrivate
 
   GnomeDesktopThumbnailFactory *thumb_factory;
 
+  GFileMonitor *picture_dir_monitor;
+  GFileMonitor *cache_dir_monitor;
+
   GHashTable *known_items;
 };
 
@@ -120,6 +124,9 @@ bg_pictures_source_finalize (GObject *object)
       bg_source->priv->known_items = NULL;
     }
 
+  g_clear_object (&bg_source->priv->picture_dir_monitor);
+  g_clear_object (&bg_source->priv->cache_dir_monitor);
+
   G_OBJECT_CLASS (bg_pictures_source_parent_class)->finalize (object);
 }
 
@@ -136,36 +143,6 @@ bg_pictures_source_class_init (BgPicturesSourceClass *klass)
   object_class->finalize = bg_pictures_source_finalize;
 }
 
-static int
-sort_func (GtkTreeModel *model,
-           GtkTreeIter *a,
-           GtkTreeIter *b,
-           BgPicturesSource *bg_source)
-{
-  CcBackgroundItem *item_a;
-  CcBackgroundItem *item_b;
-  guint64 modified_a;
-  guint64 modified_b;
-  int retval;
-
-  gtk_tree_model_get (model, a,
-                      1, &item_a,
-                      -1);
-  gtk_tree_model_get (model, b,
-                      1, &item_b,
-                      -1);
-
-  modified_a = cc_background_item_get_modified (item_a);
-  modified_b = cc_background_item_get_modified (item_b);
-
-  retval = modified_b - modified_a;
-
-  g_object_unref (item_a);
-  g_object_unref (item_b);
-
-  return retval;
-}
-
 static void
 picture_scaled (GObject *source_object,
                 GAsyncResult *res,
@@ -175,8 +152,8 @@ picture_scaled (GObject *source_object,
   CcBackgroundItem *item;
   GError *error = NULL;
   GdkPixbuf *pixbuf;
-  const char *source_url;
   const char *software;
+  const char *uri;
   GtkTreeIter iter;
   GtkListStore *store;
 
@@ -196,16 +173,7 @@ picture_scaled (GObject *source_object,
   bg_source = BG_PICTURES_SOURCE (user_data);
   store = bg_source_get_liststore (BG_SOURCE (bg_source));
   item = g_object_get_data (source_object, "item");
-
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (store),
-                                   1,
-                                   (GtkTreeIterCompareFunc)sort_func,
-                                   bg_source,
-                                   NULL);
-
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
-                                        1,
-                                        GTK_SORT_ASCENDING);
+  uri = cc_background_item_get_uri (item);
 
   /* Ignore screenshots */
   software = gdk_pixbuf_get_option (pixbuf, "tEXt::Software");
@@ -222,38 +190,15 @@ picture_scaled (GObject *source_object,
   cc_background_item_load (item, NULL);
 
   /* insert the item into the liststore */
-  gtk_list_store_insert_with_values (store, &iter, 0,
+  gtk_list_store_insert_with_values (store, &iter, -1,
                                      0, pixbuf,
                                      1, item,
                                      -1);
-  source_url = cc_background_item_get_source_url (item);
-  if (source_url != NULL)
-    {
-      g_hash_table_insert (bg_source->priv->known_items,
-			   bg_pictures_source_get_unique_filename (source_url), GINT_TO_POINTER (TRUE));
-    }
-  else
-    {
-      char *cache_path;
-      GFile *file, *parent, *dir;
 
-      cache_path = bg_pictures_source_get_cache_path ();
-      dir = g_file_new_for_path (cache_path);
-      g_free (cache_path);
+  g_hash_table_insert (bg_source->priv->known_items,
+                       bg_pictures_source_get_unique_filename (uri),
+                       GINT_TO_POINTER (TRUE));
 
-      file = g_file_new_for_uri (cc_background_item_get_uri (item));
-      parent = g_file_get_parent (file);
-
-      if (g_file_equal (parent, dir))
-        {
-          char *basename;
-          basename = g_file_get_basename (file);
-	  g_hash_table_insert (bg_source->priv->known_items,
-			       basename, GINT_TO_POINTER (TRUE));
-	}
-      g_object_unref (file);
-      g_object_unref (parent);
-    }
 
   g_object_unref (pixbuf);
 }
@@ -317,6 +262,7 @@ add_single_file (BgPicturesSource *bg_source,
   const gchar *content_type;
   CcBackgroundItem *item;
   char *uri;
+  guint64 mtime;
 
   /* find png and jpeg files */
   content_type = g_file_info_get_content_type (info);
@@ -328,17 +274,21 @@ add_single_file (BgPicturesSource *bg_source,
 
   /* create a new CcBackgroundItem */
   uri = g_file_get_uri (file);
+  mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+
   item = cc_background_item_new (uri);
   g_free (uri);
   g_object_set (G_OBJECT (item),
 		"flags", CC_BACKGROUND_ITEM_HAS_URI | CC_BACKGROUND_ITEM_HAS_SHADING,
 		"shading", G_DESKTOP_BACKGROUND_SHADING_SOLID,
 		"placement", G_DESKTOP_BACKGROUND_STYLE_ZOOM,
+                "modified", mtime,
 		NULL);
   if (source_uri != NULL)
     g_object_set (G_OBJECT (item), "source-url", source_uri, NULL);
 
   g_object_set_data (G_OBJECT (file), "item", item);
+
   g_file_read_async (file, G_PRIORITY_DEFAULT,
                      bg_source->priv->cancellable,
                      picture_opened_for_read, bg_source);
@@ -366,17 +316,15 @@ bg_pictures_source_add (BgPicturesSource *bg_source,
 
 gboolean
 bg_pictures_source_remove (BgPicturesSource *bg_source,
-			   CcBackgroundItem *item)
+                           const char       *uri)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
   gboolean cont;
-  const char *uri;
   gboolean retval;
 
   retval = FALSE;
   model = GTK_TREE_MODEL (bg_source_get_liststore (BG_SOURCE (bg_source)));
-  uri = cc_background_item_get_uri (item);
 
   cont = gtk_tree_model_get_iter_first (model, &iter);
   while (cont)
@@ -388,18 +336,13 @@ bg_pictures_source_remove (BgPicturesSource *bg_source,
       tmp_uri = cc_background_item_get_uri (tmp_item);
       if (g_str_equal (tmp_uri, uri))
         {
-          GFile *file;
           char *uuid;
-
-          file = g_file_new_for_uri (uri);
-          uuid = g_file_get_basename (file);
+          uuid = bg_pictures_source_get_unique_filename (uri);
           g_hash_table_insert (bg_source->priv->known_items,
 			       uuid, NULL);
 
           gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
           retval = TRUE;
-          g_file_trash (file, NULL, NULL);
-          g_object_unref (file);
           break;
         }
       g_object_unref (tmp_item);
@@ -408,6 +351,21 @@ bg_pictures_source_remove (BgPicturesSource *bg_source,
   return retval;
 }
 
+static int
+file_sort_func (gconstpointer a,
+                gconstpointer b)
+{
+  GFileInfo *file_a = G_FILE_INFO (a);
+  GFileInfo *file_b = G_FILE_INFO (b);
+  guint64 modified_a, modified_b;
+
+  modified_a = g_file_info_get_attribute_uint64 (file_a, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+
+  modified_b = g_file_info_get_attribute_uint64 (file_b, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+
+  return modified_b - modified_a;
+}
+
 static void
 file_info_async_ready (GObject      *source,
                        GAsyncResult *res,
@@ -417,10 +375,10 @@ file_info_async_ready (GObject      *source,
   GList *files, *l;
   GError *err = NULL;
   GFile *parent;
+
   files = g_file_enumerator_next_files_finish (G_FILE_ENUMERATOR (source),
                                                res,
                                                &err);
-
   if (err)
     {
       g_warning ("Could not get pictures file information: %s", err->message);
@@ -433,6 +391,8 @@ file_info_async_ready (GObject      *source,
 
   parent = g_file_enumerator_get_container (G_FILE_ENUMERATOR (source));
 
+  files = g_list_sort (files, file_sort_func);
+
   /* iterate over the available files */
   for (l = files; l; l = g_list_next (l))
     {
@@ -534,6 +494,109 @@ bg_pictures_source_is_known (BgPicturesSource *bg_source,
   return retval;
 }
 
+static int
+sort_func (GtkTreeModel *model,
+           GtkTreeIter *a,
+           GtkTreeIter *b,
+           BgPicturesSource *bg_source)
+{
+  CcBackgroundItem *item_a;
+  CcBackgroundItem *item_b;
+  guint64 modified_a;
+  guint64 modified_b;
+  int retval;
+
+  gtk_tree_model_get (model, a,
+                      1, &item_a,
+                      -1);
+  gtk_tree_model_get (model, b,
+                      1, &item_b,
+                      -1);
+
+  modified_a = cc_background_item_get_modified (item_a);
+  modified_b = cc_background_item_get_modified (item_b);
+
+  retval = modified_b - modified_a;
+
+  g_object_unref (item_a);
+  g_object_unref (item_b);
+
+  return retval;
+}
+
+static void
+file_info_ready (GObject      *object,
+                 GAsyncResult *res,
+                 gpointer      user_data)
+{
+  GFileInfo *info;
+  GError *error = NULL;
+  GFile *file = G_FILE (object);
+
+  info = g_file_query_info_finish (file, res, &error);
+
+  if (!info)
+    {
+      g_warning ("Problem looking up file info: %s", error->message);
+      g_clear_error (&error);
+      return;
+    }
+
+  /* Up the ref count so we can re-use the add_single_item code path which
+   * reduces the ref count.
+   */
+  g_object_ref (file);
+  add_single_file (BG_PICTURES_SOURCE (user_data), file, info, NULL);
+}
+
+static void
+file_added (GFile            *file,
+            BgPicturesSource *self)
+{
+  char *uri;
+  uri = g_file_get_uri (file);
+
+  if (!bg_pictures_source_is_known (self, uri))
+    {
+      g_file_query_info_async (file,
+                               ATTRIBUTES,
+                               G_FILE_QUERY_INFO_NONE,
+                               G_PRIORITY_LOW,
+                               NULL,
+                               file_info_ready,
+                               self);
+    }
+
+  g_free (uri);
+}
+
+static void
+files_changed_cb (GFileMonitor      *monitor,
+                  GFile             *file,
+                  GFile             *other_file,
+                  GFileMonitorEvent  event_type,
+                  gpointer           user_data)
+{
+  BgPicturesSource *self = BG_PICTURES_SOURCE (user_data);
+  char *uri;
+
+  switch (event_type)
+    {
+      case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+        file_added (file, self);
+        break;
+
+      case G_FILE_MONITOR_EVENT_DELETED:
+        uri = g_file_get_uri (file);
+        bg_pictures_source_remove (self, uri);
+        g_free (uri);
+        break;
+
+      default:
+        return;
+    }
+}
+
 static void
 bg_pictures_source_init (BgPicturesSource *self)
 {
@@ -541,6 +604,7 @@ bg_pictures_source_init (BgPicturesSource *self)
   BgPicturesSourcePrivate *priv;
   GFile *dir;
   char *cache_path;
+  GtkListStore *store;
 
   priv = self->priv = PICTURES_SOURCE_PRIVATE (self);
 
@@ -557,6 +621,18 @@ bg_pictures_source_init (BgPicturesSource *self)
                                    G_FILE_QUERY_INFO_NONE,
                                    G_PRIORITY_LOW, priv->cancellable,
                                    dir_enum_async_ready, self);
+
+  priv->picture_dir_monitor = g_file_monitor_directory (dir,
+                                                        G_FILE_MONITOR_NONE,
+                                                        priv->cancellable,
+                                                        NULL);
+
+  if (priv->picture_dir_monitor)
+    g_signal_connect (priv->picture_dir_monitor,
+                      "changed",
+                      G_CALLBACK (files_changed_cb),
+                      self);
+
   g_object_unref (dir);
 
   cache_path = bg_pictures_source_get_cache_path ();
@@ -566,10 +642,33 @@ bg_pictures_source_init (BgPicturesSource *self)
                                    G_FILE_QUERY_INFO_NONE,
                                    G_PRIORITY_LOW, priv->cancellable,
                                    dir_enum_async_ready, self);
+
+  priv->cache_dir_monitor = g_file_monitor_directory (dir,
+                                                      G_FILE_MONITOR_NONE,
+                                                      priv->cancellable,
+                                                      NULL);
+  if (priv->cache_dir_monitor)
+    g_signal_connect (priv->cache_dir_monitor,
+                      "changed",
+                      G_CALLBACK (files_changed_cb),
+                      self);
+
   g_object_unref (dir);
 
   priv->thumb_factory =
     gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+
+  store = bg_source_get_liststore (BG_SOURCE (self));
+
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (store),
+                                   1,
+                                   (GtkTreeIterCompareFunc)sort_func,
+                                   self,
+                                   NULL);
+
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+                                        1,
+                                        GTK_SORT_ASCENDING);
 }
 
 BgPicturesSource *
