@@ -243,20 +243,21 @@ on_screenshot_finished (GObject *source,
 {
   ScreenshotData *data = user_data;
   CcBackgroundPanel *panel = data->panel;
-  CcBackgroundPanelPrivate *priv = panel->priv;
+  CcBackgroundPanelPrivate *priv;
   GError *error;
   GdkPixbuf *pixbuf;
   cairo_surface_t *surface;
   cairo_t *cr;
+  GVariant *result;
   int width;
   int height;
 
   error = NULL;
-  g_dbus_connection_call_finish (G_DBUS_CONNECTION (source),
-                                 res,
-                                 &error);
+  result = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source),
+                                          res,
+                                          &error);
 
-  if (error != NULL) {
+  if (result == NULL) {
     if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
       g_error_free (error);
       return;
@@ -267,9 +268,12 @@ on_screenshot_finished (GObject *source,
     /* fallback? */
     goto out;
   }
+  g_variant_unref (result);
+
+  priv = panel->priv;
 
   pixbuf = gdk_pixbuf_new_from_file (panel->priv->screenshot_path, &error);
-  if (error != NULL)
+  if (pixbuf == NULL)
     {
       g_debug ("Unable to use GNOME Shell's builtin screenshot interface: %s",
                error->message);
@@ -382,9 +386,9 @@ get_screenshot_async (CcBackgroundPanel *panel)
                                  panel->priv->screenshot_path);
 
   g_dbus_connection_call (panel->priv->connection,
-                          "org.gnome.Shell",
-                          "/org/gnome/Shell",
-                          "org.gnome.Shell",
+                          "org.gnome.Shell.Screenshot",
+                          "/org/gnome/Shell/Screenshot",
+                          "org.gnome.Shell.Screenshot",
                           method_name,
                           method_params,
                           NULL,
