@@ -84,6 +84,14 @@ method_changed (GtkComboBox *combo, CEPageIP4 *page)
         ce_page_changed (CE_PAGE (page));
 }
 
+static void
+switch_toggled (GObject    *object,
+                GParamSpec *pspec,
+                CEPage     *page)
+{
+        ce_page_changed (page);
+}
+
 static void
 update_separator (GtkWidget **separator,
                   GtkWidget  *child,
@@ -173,20 +181,16 @@ add_address_row (CEPageIP4   *page,
 {
         GtkWidget *row;
         GtkWidget *widget;
+        GtkWidget *label;
         GtkWidget *delete_button;
         GtkWidget *image;
 
         row = gtk_grid_new ();
-        widget = gtk_label_new (_("Address"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_grid_attach (GTK_GRID (row), widget, 1, 1, 1, 1);
-        widget = gtk_label_new (_("Netmask"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_grid_attach (GTK_GRID (row), widget, 1, 2, 1, 1);
-        widget = gtk_label_new (_("Gateway"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_grid_attach (GTK_GRID (row), widget, 1, 3, 1, 1);
+        label = gtk_label_new (_("Address"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_grid_attach (GTK_GRID (row), label, 1, 1, 1, 1);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "address", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), address);
@@ -194,7 +198,12 @@ add_address_row (CEPageIP4   *page,
         gtk_widget_set_margin_right (widget, 10);
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_grid_attach (GTK_GRID (row), widget, 2, 1, 1, 1);
+
+        label = gtk_label_new (_("Netmask"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_grid_attach (GTK_GRID (row), label, 1, 2, 1, 1);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "network", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), network);
@@ -202,7 +211,13 @@ add_address_row (CEPageIP4   *page,
         gtk_widget_set_margin_right (widget, 10);
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_grid_attach (GTK_GRID (row), widget, 2, 2, 1, 1);
+
+
+        label = gtk_label_new (_("Gateway"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_grid_attach (GTK_GRID (row), label, 1, 3, 1, 1);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), gateway);
@@ -212,12 +227,15 @@ add_address_row (CEPageIP4   *page,
         gtk_grid_attach (GTK_GRID (row), widget, 2, 3, 1, 1);
 
         delete_button = gtk_button_new ();
+        gtk_style_context_add_class (gtk_widget_get_style_context (delete_button), "image-button");
         g_signal_connect (delete_button, "clicked", G_CALLBACK (remove_row), page);
         image = gtk_image_new_from_icon_name ("user-trash-symbolic", GTK_ICON_SIZE_MENU);
+        atk_object_set_name (gtk_widget_get_accessible (delete_button), _("Delete Address"));
         gtk_button_set_image (GTK_BUTTON (delete_button), image);
         gtk_grid_attach (GTK_GRID (row), delete_button, 3, 2, 1, 1);
         g_object_set_data (G_OBJECT (row), "delete-button", delete_button);
 
+        gtk_grid_set_row_spacing (GTK_GRID (row), 10);
         gtk_widget_set_margin_left (row, 10);
         gtk_widget_set_margin_right (row, 10);
         gtk_widget_set_margin_top (row, 10);
@@ -264,6 +282,7 @@ add_section_toolbar (CEPageIP4 *page, GtkWidget *section, GCallback add_cb)
         button = gtk_button_new ();
         g_signal_connect_swapped (button, "clicked", G_CALLBACK (add_cb), page);
         image = gtk_image_new_from_icon_name ("list-add-symbolic", GTK_ICON_SIZE_MENU);
+        atk_object_set_name (gtk_widget_get_accessible (button), _("Add"));
         gtk_button_set_image (GTK_BUTTON (button), image);
         gtk_container_add (GTK_CONTAINER (box), button);
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 1);
@@ -322,15 +341,17 @@ add_dns_row (CEPageIP4   *page,
              const gchar *address)
 {
         GtkWidget *row;
+        GtkWidget *label;
         GtkWidget *widget;
         GtkWidget *delete_button;
         GtkWidget *image;
 
         row = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-        widget = gtk_label_new (_("Server"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_box_pack_start (GTK_BOX (row), widget, FALSE, FALSE, 0);
+        label = gtk_label_new (_("Server"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_box_pack_start (GTK_BOX (row), label, FALSE, FALSE, 0);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "address", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), address);
@@ -340,8 +361,10 @@ add_dns_row (CEPageIP4   *page,
         gtk_box_pack_start (GTK_BOX (row), widget, TRUE, TRUE, 0);
 
         delete_button = gtk_button_new ();
+        gtk_style_context_add_class (gtk_widget_get_style_context (delete_button), "image-button");
         g_signal_connect (delete_button, "clicked", G_CALLBACK (remove_row), page);
         image = gtk_image_new_from_icon_name ("user-trash-symbolic", GTK_ICON_SIZE_MENU);
+        atk_object_set_name (gtk_widget_get_accessible (delete_button), _("Delete DNS Server"));
         gtk_button_set_image (GTK_BUTTON (delete_button), image);
         gtk_box_pack_start (GTK_BOX (row), delete_button, FALSE, FALSE, 0);
         g_object_set_data (G_OBJECT (row), "delete-button", delete_button);
@@ -382,6 +405,7 @@ add_dns_section (CEPageIP4 *page)
         gtk_container_add (GTK_CONTAINER (frame), list);
         page->auto_dns = GTK_SWITCH (gtk_builder_get_object (CE_PAGE (page)->builder, "auto_dns_switch"));
         gtk_switch_set_active (page->auto_dns, !nm_setting_ip4_config_get_ignore_auto_dns (page->setting));
+        g_signal_connect (page->auto_dns, "notify::active", G_CALLBACK (switch_toggled), page);
 
         add_section_toolbar (page, widget, G_CALLBACK (add_empty_dns_row));
 
@@ -408,25 +432,17 @@ add_route_row (CEPageIP4   *page,
                gint         metric)
 {
         GtkWidget *row;
+        GtkWidget *label;
         GtkWidget *widget;
         GtkWidget *delete_button;
         GtkWidget *image;
 
         row = gtk_grid_new ();
-        widget = gtk_label_new (_("Address"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_grid_attach (GTK_GRID (row), widget, 1, 1, 1, 1);
-        widget = gtk_label_new (_("Netmask"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_grid_attach (GTK_GRID (row), widget, 1, 2, 1, 1);
-        widget = gtk_label_new (_("Gateway"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_grid_attach (GTK_GRID (row), widget, 1, 3, 1, 1);
-        /* Translators: Please see http://en.wikipedia.org/wiki/Metrics_(networking) */
-        widget = gtk_label_new (C_("network parameters", "Metric"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
-        gtk_grid_attach (GTK_GRID (row), widget, 1, 4, 1, 1);
+        label = gtk_label_new (_("Address"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_grid_attach (GTK_GRID (row), label, 1, 1, 1, 1);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "address", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), address);
@@ -434,7 +450,12 @@ add_route_row (CEPageIP4   *page,
         gtk_widget_set_margin_right (widget, 10);
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_grid_attach (GTK_GRID (row), widget, 2, 1, 1, 1);
+
+        label = gtk_label_new (_("Netmask"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_grid_attach (GTK_GRID (row), label, 1, 2, 1, 1);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "netmask", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), netmask);
@@ -442,7 +463,12 @@ add_route_row (CEPageIP4   *page,
         gtk_widget_set_margin_right (widget, 10);
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_grid_attach (GTK_GRID (row), widget, 2, 2, 1, 1);
+
+        label = gtk_label_new (_("Gateway"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_grid_attach (GTK_GRID (row), label, 1, 3, 1, 1);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), gateway);
@@ -450,7 +476,13 @@ add_route_row (CEPageIP4   *page,
         gtk_widget_set_margin_right (widget, 10);
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_grid_attach (GTK_GRID (row), widget, 2, 3, 1, 1);
+
+        /* Translators: Please see https://en.wikipedia.org/wiki/Metrics_(networking) */
+        label = gtk_label_new (C_("network parameters", "Metric"));
+        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+        gtk_grid_attach (GTK_GRID (row), label, 1, 4, 1, 1);
         widget = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
         g_object_set_data (G_OBJECT (row), "metric", widget);
         if (metric > 0) {
@@ -464,14 +496,17 @@ add_route_row (CEPageIP4   *page,
         gtk_grid_attach (GTK_GRID (row), widget, 2, 4, 1, 1);
 
         delete_button = gtk_button_new ();
+        gtk_style_context_add_class (gtk_widget_get_style_context (delete_button), "image-button");
         g_signal_connect (delete_button, "clicked", G_CALLBACK (remove_row), page);
         image = gtk_image_new_from_icon_name ("user-trash-symbolic", GTK_ICON_SIZE_MENU);
+        atk_object_set_name (gtk_widget_get_accessible (delete_button), _("Delete Route"));
         gtk_button_set_image (GTK_BUTTON (delete_button), image);
         gtk_widget_set_halign (delete_button, GTK_ALIGN_CENTER);
         gtk_widget_set_valign (delete_button, GTK_ALIGN_CENTER);
         gtk_grid_attach (GTK_GRID (row), delete_button, 3, 1, 1, 4);
         g_object_set_data (G_OBJECT (row), "delete-button", delete_button);
 
+        gtk_grid_set_row_spacing (GTK_GRID (row), 10);
         gtk_widget_set_margin_left (row, 10);
         gtk_widget_set_margin_right (row, 10);
         gtk_widget_set_margin_top (row, 10);
@@ -509,6 +544,7 @@ add_routes_section (CEPageIP4 *page)
         gtk_container_add (GTK_CONTAINER (frame), list);
         page->auto_routes = GTK_SWITCH (gtk_builder_get_object (CE_PAGE (page)->builder, "auto_routes_switch"));
         gtk_switch_set_active (page->auto_routes, !nm_setting_ip4_config_get_ignore_auto_routes (page->setting));
+        g_signal_connect (page->auto_routes, "notify::active", G_CALLBACK (switch_toggled), page);
 
         add_section_toolbar (page, widget, G_CALLBACK (add_empty_route_row));
 
@@ -562,6 +598,7 @@ connect_ip4_page (CEPageIP4 *page)
         add_routes_section (page);
 
         page->enabled = GTK_SWITCH (gtk_builder_get_object (CE_PAGE (page)->builder, "switch_enable"));
+        g_signal_connect (page->enabled, "notify::active", G_CALLBACK (switch_toggled), page);
 
         str_method = nm_setting_ip4_config_get_method (page->setting);
         disabled = g_strcmp0 (str_method, NM_SETTING_IP4_CONFIG_METHOD_DISABLED) == 0;
