@@ -494,7 +494,6 @@ nm_device_wifi_refresh_ui (NetDeviceWifi *device_wifi)
         const gchar *str;
         gboolean is_hotspot;
         gchar *str_tmp = NULL;
-        GtkWidget *widget;
         gint strength = 0;
         guint speed = 0;
         NMAccessPoint *active_ap;
@@ -595,9 +594,7 @@ nm_device_wifi_refresh_ui (NetDeviceWifi *device_wifi)
         else
                 panel_set_device_widget_details (priv->builder, "last_used", NULL);
 
-        widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder, "heading_status"));
-        gtk_label_set_label (GTK_LABEL (widget),
-                             panel_device_state_to_localized_string (nm_device));
+        panel_set_device_status (priv->builder, "heading_status", nm_device, NULL);
 
         /* update list of APs */
         populate_ap_list (device_wifi);
@@ -1018,12 +1015,10 @@ start_shared_connection (NetDeviceWifi *device_wifi)
 
         sw = (NMSettingWireless *)nm_setting_wireless_new ();
 
-#ifdef HAVE_NM_UNSTABLE
 	/* Use real AP mode if the device supports it */
         if (nm_device_wifi_get_capabilities (NM_DEVICE_WIFI (device)) & NM_WIFI_DEVICE_CAP_AP)
 		mode = NM_SETTING_WIRELESS_MODE_AP;
         else
-#endif
                 mode = NM_SETTING_WIRELESS_MODE_ADHOC;
 
         g_object_set (sw,
@@ -1260,6 +1255,8 @@ net_device_wifi_constructed (GObject *object)
         NMClient *client;
         NMRemoteSettings *remote_settings;
         NMClientPermissionResult perm;
+        NMDevice *nm_device;
+        NMDeviceWifiCapabilities caps;
         GtkWidget *widget;
 
         G_OBJECT_CLASS (net_device_wifi_parent_class)->constructed (object);
@@ -1268,12 +1265,22 @@ net_device_wifi_constructed (GObject *object)
         g_signal_connect (client, "notify::wireless-enabled",
                           G_CALLBACK (wireless_enabled_toggled), device_wifi);
 
-        /* only show the button if the user can create a hotspot */
-        perm = nm_client_get_permission_result (client, NM_CLIENT_PERMISSION_WIFI_SHARE_OPEN);
+        nm_device = net_device_get_nm_device (NET_DEVICE (device_wifi));
+
+        /* only enable the button if the user can create a hotspot */
         widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
                                                      "start_hotspot_button"));
-        gtk_widget_set_sensitive (widget, perm == NM_CLIENT_PERMISSION_RESULT_YES ||
-                                          perm == NM_CLIENT_PERMISSION_RESULT_AUTH);
+        perm = nm_client_get_permission_result (client, NM_CLIENT_PERMISSION_WIFI_SHARE_OPEN);
+        caps = nm_device_wifi_get_capabilities (NM_DEVICE_WIFI (nm_device));
+        if (perm != NM_CLIENT_PERMISSION_RESULT_YES &&
+            perm != NM_CLIENT_PERMISSION_RESULT_AUTH) {
+                gtk_widget_set_tooltip_text (widget, _("System policy prohibits use as a Hotspot"));
+                gtk_widget_set_sensitive (widget, FALSE);
+        } else if (!(caps & (NM_WIFI_DEVICE_CAP_AP | NM_WIFI_DEVICE_CAP_ADHOC))) {
+                gtk_widget_set_tooltip_text (widget, _("Wireless device does not support Hotspot mode"));
+                gtk_widget_set_sensitive (widget, FALSE);
+        } else
+                gtk_widget_set_sensitive (widget, TRUE);
 
         remote_settings = net_object_get_remote_settings (NET_OBJECT (device_wifi));
         g_signal_connect (remote_settings, "connections-read",
@@ -1532,12 +1539,14 @@ make_row (GtkSizeGroup   *rows,
                 image = gtk_image_new_from_icon_name ("emblem-system-symbolic", GTK_ICON_SIZE_MENU);
                 gtk_widget_show (image);
                 widget = gtk_button_new ();
+                gtk_style_context_add_class (gtk_widget_get_style_context (widget), "image-button");
                 gtk_widget_set_no_show_all (widget, TRUE);
                 if (!connecting)
                         gtk_widget_show (widget);
                 gtk_container_add (GTK_CONTAINER (widget), image);
                 gtk_widget_set_halign (widget, GTK_ALIGN_CENTER);
                 gtk_widget_set_valign (widget, GTK_ALIGN_CENTER);
+                atk_object_set_name (gtk_widget_get_accessible (widget), _("Options…"));
                 gtk_box_pack_start (GTK_BOX (row), widget, FALSE, FALSE, 0);
                 gtk_size_group_add_widget (spinner_button_group, widget);
                 g_object_set_data (G_OBJECT (row), "edit", widget);
@@ -1781,6 +1790,7 @@ open_history (NetDeviceWifi *device_wifi)
                 }
         }
         g_slist_free (connections);
+        g_ptr_array_free (aps_unique, TRUE);
 
         gtk_window_present (GTK_WINDOW (dialog));
 }
@@ -1835,8 +1845,10 @@ populate_ap_list (NetDeviceWifi *device_wifi)
                         NMSetting *setting;
                         const GByteArray *ssid;
 
-                        if (connection_is_shared (connection))
+                        if (connection_is_shared (connection)) {
+                                connection = NULL;
                                 continue;
+                        }
 
                         setting = nm_connection_get_setting_by_name (connection, NM_SETTING_WIRELESS_SETTING_NAME);
                         ssid = nm_setting_wireless_get_ssid (NM_SETTING_WIRELESS (setting));
@@ -1855,6 +1867,7 @@ populate_ap_list (NetDeviceWifi *device_wifi)
         }
 
         g_slist_free (connections);
+        g_ptr_array_free (aps_unique, TRUE);
 }
 
 static void
