@@ -409,8 +409,8 @@ printer_add_real_async_dbus_cb (GObject      *source_object,
                                 GAsyncResult *res,
                                 gpointer      user_data)
 {
-  PpNewPrinter        *printer;
-  PpNewPrinterPrivate *priv;
+  PpNewPrinter        *printer = (PpNewPrinter *) user_data;
+  PpNewPrinterPrivate *priv = printer->priv;
   GVariant            *output;
   GError              *error = NULL;
 
@@ -426,7 +426,7 @@ printer_add_real_async_dbus_cb (GObject      *source_object,
       g_variant_get (output, "(&s)", &ret_error);
       if (ret_error[0] != '\0')
         {
-          g_warning ("%s", ret_error);
+          g_warning ("cups-pk-helper: addition of printer %s failed: %s", priv->name, ret_error);
         }
 
       g_variant_unref (output);
@@ -442,9 +442,6 @@ printer_add_real_async_dbus_cb (GObject      *source_object,
       error->domain != G_IO_ERROR ||
       error->code != G_IO_ERROR_CANCELLED)
     {
-      printer = (PpNewPrinter *) user_data;
-      priv = printer->priv;
-
       get_named_dest_async (priv->name,
                             printer_add_real_async_cb,
                             printer);
@@ -1084,6 +1081,12 @@ get_missing_executables_cb (GObject      *source_object,
   GList    *executables = NULL;
   GList    *item;
 
+  if (data->ppd_file_name)
+    {
+      g_unlink (data->ppd_file_name);
+      g_clear_pointer (&data->ppd_file_name, g_free);
+    }
+
   output = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object),
                                           res,
                                           &error);
@@ -1160,12 +1163,6 @@ DBus method \"MissingExecutables\" to find missing executables and filters.");
       g_object_unref (source_object);
       install_missing_executables_cb (data);
     }
-
-  if (data->ppd_file_name)
-    {
-      g_unlink (data->ppd_file_name);
-      g_clear_pointer (&data->ppd_file_name, g_free);
-    }
 }
 
 static void
