@@ -44,6 +44,7 @@ struct _CcPrivacyPanelPrivate
   GSettings  *lockdown_settings;
   GSettings  *lock_settings;
   GSettings  *privacy_settings;
+  GSettings  *notification_settings;
 };
 
 static void
@@ -295,7 +296,7 @@ add_screen_lock (CcPrivacyPanel *self)
                     G_CALLBACK (lock_combo_changed_cb), self);
 
   w = GTK_WIDGET (gtk_builder_get_object (self->priv->builder, "show_notifications"));
-  g_settings_bind (self->priv->lock_settings, "show-notifications",
+  g_settings_bind (self->priv->notification_settings, "show-in-lock-screen",
                    w, "active",
                    G_SETTINGS_BIND_DEFAULT);
 }
@@ -613,6 +614,7 @@ cc_privacy_panel_finalize (GObject *object)
   g_clear_object (&priv->lockdown_settings);
   g_clear_object (&priv->lock_settings);
   g_clear_object (&priv->privacy_settings);
+  g_clear_object (&priv->notification_settings);
 
   G_OBJECT_CLASS (cc_privacy_panel_parent_class)->finalize (object);
 }
@@ -661,7 +663,7 @@ cc_privacy_panel_init (CcPrivacyPanel *self)
 {
   GError    *error;
   GtkWidget *widget;
-  GtkWidget *scrolled_window;
+  GtkWidget *frame;
   guint res;
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CC_TYPE_PRIVACY_PANEL, CcPrivacyPanelPrivate);
@@ -682,10 +684,10 @@ cc_privacy_panel_init (CcPrivacyPanel *self)
       return;
     }
 
-  scrolled_window = WID ("scrolled_window");
+  frame = WID ("frame");
   widget = GTK_WIDGET (egg_list_box_new ());
   egg_list_box_set_selection_mode (EGG_LIST_BOX (widget), GTK_SELECTION_NONE);
-  egg_list_box_add_to_scrolled (EGG_LIST_BOX (widget), GTK_SCROLLED_WINDOW (scrolled_window));
+  gtk_container_add (GTK_CONTAINER (frame), widget);
   self->priv->list_box = widget;
   gtk_widget_show (widget);
 
@@ -699,6 +701,7 @@ cc_privacy_panel_init (CcPrivacyPanel *self)
   self->priv->lockdown_settings = g_settings_new ("org.gnome.desktop.lockdown");
   self->priv->lock_settings = g_settings_new ("org.gnome.desktop.screensaver");
   self->priv->privacy_settings = g_settings_new ("org.gnome.desktop.privacy");
+  self->priv->notification_settings = g_settings_new ("org.gnome.desktop.notifications");
 
   add_screen_lock (self);
   add_name_visibility (self);
