@@ -20,6 +20,7 @@
  */
 
 #include <config.h>
+#include <locale.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gio/gdesktopappinfo.h>
@@ -59,6 +60,8 @@
 #define INPUT_SOURCE_TYPE_XKB "xkb"
 #define INPUT_SOURCE_TYPE_IBUS "ibus"
 
+#define MAX_INPUT_ROWS_VISIBLE 5
+
 CC_PANEL_REGISTER (CcRegionPanel, cc_region_panel)
 
 #define WID(s) GTK_WIDGET (gtk_builder_get_object (self->priv->builder, s))
@@ -80,6 +83,8 @@ struct _CcRegionPanelPrivate {
         GPermission *permission;
         SystemOp     op;
         GDBusProxy  *localed;
+        GDBusProxy  *session;
+        GCancellable *cancellable;
 
         GtkWidget *overlay;
         GtkWidget *notification;
@@ -106,6 +111,8 @@ struct _CcRegionPanelPrivate {
         GtkWidget *remove_input;
         GtkWidget *show_config;
         GtkWidget *show_layout;
+        GtkWidget *input_scrolledwindow;
+        guint n_input_rows;
 
         GSettings *input_settings;
         GnomeXkbInfo *xkb_info;
@@ -122,6 +129,9 @@ cc_region_panel_finalize (GObject *object)
 	CcRegionPanel *self = CC_REGION_PANEL (object);
 	CcRegionPanelPrivate *priv = self->priv;
 
+        g_cancellable_cancel (priv->cancellable);
+        g_clear_object (&priv->cancellable);
+
         if (priv->user_manager) {
                 g_signal_handlers_disconnect_by_data (priv->user_manager, self);
                 priv->user_manager = NULL;
@@ -134,6 +144,7 @@ cc_region_panel_finalize (GObject *object)
 
         g_clear_object (&priv->permission);
         g_clear_object (&priv->localed);
+        g_clear_object (&priv->session);
         g_clear_object (&priv->builder);
         g_clear_object (&priv->locale_settings);
         g_clear_object (&priv->input_settings);
@@ -188,32 +199,35 @@ cc_region_panel_class_init (CcRegionPanelClass * klass)
 static void
 restart_now (CcRegionPanel *self)
 {
-        GDBusConnection *bus;
+        CcRegionPanelPrivate *priv = self->priv;
 
         gd_notification_dismiss (GD_NOTIFICATION (self->priv->notification));
 
-        bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
-        g_dbus_connection_call (bus,
-                                "org.gnome.SessionManager",
-                                "/org/gnome/SessionManager",
-                                "org.gnome.SessionManager",
-                                "Logout",
-                                g_variant_new ("(u)", 0),
-                                NULL, 0, G_MAXINT,
-                                NULL, NULL, NULL);
+        g_dbus_proxy_call (priv->session,
+                           "Logout",
+                           g_variant_new ("(u)", 0),
+                           G_DBUS_CALL_FLAGS_NONE,
+                           -1, NULL, NULL, NULL);
 }
 
 static void
-show_restart_notification (CcRegionPanel *self)
+show_restart_notification (CcRegionPanel *self,
+                           const gchar   *locale)
 {
 	CcRegionPanelPrivate *priv = self->priv;
         GtkWidget *box;
         GtkWidget *label;
         GtkWidget *button;
+        gchar *current_locale;
 
         if (priv->notification)
                 return;
 
+        if (locale) {
+                current_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
+                setlocale (LC_MESSAGES, locale);
+        }
+
         priv->notification = gd_notification_new ();
         g_object_add_weak_pointer (G_OBJECT (priv->notification),
                                    (gpointer *)&priv->notification);
@@ -233,6 +247,11 @@ show_restart_notification (CcRegionPanel *self)
         gtk_container_add (GTK_CONTAINER (priv->notification), box);
         gtk_overlay_add_overlay (GTK_OVERLAY (self->priv->overlay), priv->notification);
         gtk_widget_show (priv->notification);
+
+        if (locale) {
+                setlocale (LC_MESSAGES, current_locale);
+                g_free (current_locale);
+        }
 }
 
 static void
@@ -251,10 +270,91 @@ update_separator_func (GtkWidget **separator,
         }
 }
 
-static void set_localed_locale (CcRegionPanel *self,
-                                const gchar   *language);
+typedef struct {
+        CcRegionPanel *self;
+        int category;
+        gchar *target_locale;
+} MaybeNotifyData;
 
-static gboolean
+static void
+maybe_notify_finish (GObject      *source,
+                     GAsyncResult *res,
+                     gpointer      data)
+{
+        MaybeNotifyData *mnd = data;
+        CcRegionPanel *self = mnd->self;
+        GError *error = NULL;
+        GVariant *retval = NULL;
+        gchar *current_lang_code = NULL;
+        gchar *current_country_code = NULL;
+        gchar *target_lang_code = NULL;
+        gchar *target_country_code = NULL;
+        const gchar *current_locale = NULL;
+
+        retval = g_dbus_proxy_call_finish (G_DBUS_PROXY (source), res, &error);
+        if (!retval) {
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Failed to get locale: %s\n", error->message);
+                goto out;
+        }
+
+        g_variant_get (retval, "(&s)", &current_locale);
+
+        if (!gnome_parse_locale (current_locale,
+                                 &current_lang_code,
+                                 &current_country_code,
+                                 NULL,
+                                 NULL))
+                goto out;
+
+        if (!gnome_parse_locale (mnd->target_locale,
+                                 &target_lang_code,
+                                 &target_country_code,
+                                 NULL,
+                                 NULL))
+                goto out;
+
+        if (g_str_equal (current_lang_code, target_lang_code) == FALSE ||
+            g_str_equal (current_country_code, target_country_code) == FALSE)
+                show_restart_notification (self,
+                                           mnd->category == LC_MESSAGES ? mnd->target_locale : NULL);
+out:
+        g_free (target_country_code);
+        g_free (target_lang_code);
+        g_free (current_country_code);
+        g_free (current_lang_code);
+        g_clear_pointer (&retval, g_variant_unref);
+        g_clear_error (&error);
+        g_free (mnd->target_locale);
+        g_free (mnd);
+}
+
+static void
+maybe_notify (CcRegionPanel *self,
+              int            category,
+              const gchar   *target_locale)
+{
+        CcRegionPanelPrivate *priv = self->priv;
+        MaybeNotifyData *mnd;
+
+        mnd = g_new0 (MaybeNotifyData, 1);
+        mnd->self = self;
+        mnd->category = category;
+        mnd->target_locale = g_strdup (target_locale);
+
+        g_dbus_proxy_call (priv->session,
+                           "GetLocale",
+                           g_variant_new ("(i)", category),
+                           G_DBUS_CALL_FLAGS_NONE,
+                           -1,
+                           priv->cancellable,
+                           maybe_notify_finish,
+                           mnd);
+}
+
+static void set_localed_locale (CcRegionPanel *self);
+
+static void
 update_language (CcRegionPanel *self,
                  const gchar   *language)
 {
@@ -262,14 +362,13 @@ update_language (CcRegionPanel *self,
 
         if (priv->login) {
                 if (g_strcmp0 (language, priv->system_language) == 0)
-                        return FALSE;
-                set_localed_locale (self, language);
-                return FALSE; /* don't show notification for login */
+                        return;
+                set_localed_locale (self);
         } else {
                 if (g_strcmp0 (language, priv->language) == 0)
-                        return FALSE;
+                        return;
                 act_user_set_language (priv->user, language);
-                return TRUE;
+                maybe_notify (self, LC_MESSAGES, language);
         }
 }
 
@@ -279,32 +378,26 @@ language_response (GtkDialog     *chooser,
                    CcRegionPanel *self)
 {
         const gchar *language;
-        gboolean changed;
-
-        changed = FALSE;
 
         if (response_id == GTK_RESPONSE_OK) {
                 language = cc_language_chooser_get_language (GTK_WIDGET (chooser));
-                changed = update_language (self, language);
+                update_language (self, language);
         }
 
         gtk_widget_destroy (GTK_WIDGET (chooser));
-
-        if (changed)
-                show_restart_notification (self);
 }
 
-static gboolean
+static void
 update_region (CcRegionPanel *self,
                const gchar   *region)
 {
 	CcRegionPanelPrivate *priv = self->priv;
 
         if (g_strcmp0 (region, priv->region) == 0)
-                return FALSE;
+                return;
 
         g_settings_set_string (priv->locale_settings, KEY_REGION, region);
-        return TRUE;
+        maybe_notify (self, LC_TIME, region);
 }
 
 static void
@@ -313,19 +406,13 @@ format_response (GtkDialog *chooser,
                  CcRegionPanel *self)
 {
         const gchar *region;
-        gboolean changed;
-
-        changed = FALSE;
 
         if (response_id == GTK_RESPONSE_OK) {
                 region = cc_format_chooser_get_region (GTK_WIDGET (chooser));
-                changed = update_region (self, region);
+                update_region (self, region);
         }
 
         gtk_widget_destroy (GTK_WIDGET (chooser));
-
-        if (changed)
-                show_restart_notification (self);
 }
 
 static void
@@ -419,6 +506,33 @@ activate_language_child (CcRegionPanel *self, GtkWidget *child)
         }
 }
 
+static void
+update_region_label (CcRegionPanel *self)
+{
+        CcRegionPanelPrivate *priv = self->priv;
+        const gchar *region;
+        gchar *name;
+
+        if (priv->region == NULL || priv->region[0] == '\0')
+                region = priv->language;
+        else
+                region = priv->region;
+
+        name = gnome_get_country_from_locale (region, region);
+        gtk_label_set_label (GTK_LABEL (priv->formats_label), name);
+        g_free (name);
+}
+
+static void
+update_region_from_setting (CcRegionPanel *self)
+{
+        CcRegionPanelPrivate *priv = self->priv;
+
+        g_free (priv->region);
+        priv->region = g_settings_get_string (priv->locale_settings, KEY_REGION);
+        update_region_label (self);
+}
+
 static void
 update_language_label (CcRegionPanel *self)
 {
@@ -436,6 +550,9 @@ update_language_label (CcRegionPanel *self)
                 name = g_strdup (C_("Language", "None"));
         gtk_label_set_label (GTK_LABEL (priv->language_label), name);
         g_free (name);
+
+        /* Formats will change too if not explicitly set. */
+        update_region_label (self);
 }
 
 static void
@@ -454,19 +571,6 @@ update_language_from_user (CcRegionPanel *self)
         update_language_label (self);
 }
 
-static void
-update_region_from_setting (CcRegionPanel *self)
-{
-	CcRegionPanelPrivate *priv = self->priv;
-        gchar *name;
-
-        g_free (priv->region);
-        priv->region = g_settings_get_string (priv->locale_settings, KEY_REGION);
-        name = gnome_get_country_from_locale (priv->region, priv->region);
-        gtk_label_set_label (GTK_LABEL (priv->formats_label), name);
-        g_free (name);
-}
-
 static void
 setup_language_section (CcRegionPanel *self)
 {
@@ -498,8 +602,8 @@ setup_language_section (CcRegionPanel *self)
         g_signal_connect_swapped (widget, "child-activated",
                                   G_CALLBACK (activate_language_child), self);
 
-        update_region_from_setting (self);
         update_language_from_user (self);
+        update_region_from_setting (self);
 }
 
 #ifdef HAVE_IBUS
@@ -634,6 +738,28 @@ setup_app_info_for_id (const gchar *id)
 }
 #endif
 
+static void
+adjust_input_list_scrolling (CcRegionPanel *self)
+{
+        CcRegionPanelPrivate *priv = self->priv;
+
+        if (priv->n_input_rows >= MAX_INPUT_ROWS_VISIBLE) {
+                GtkWidget *parent;
+                gint height;
+
+                parent = gtk_widget_get_parent (priv->input_scrolledwindow);
+                gtk_widget_get_preferred_height (parent, NULL, &height);
+                gtk_widget_set_size_request (parent, -1, height);
+
+                gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->input_scrolledwindow),
+                                                GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+        } else {
+                gtk_widget_set_size_request (gtk_widget_get_parent (priv->input_scrolledwindow), -1, -1);
+                gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->input_scrolledwindow),
+                                                GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+        }
+}
+
 static GtkWidget *
 add_input_row (CcRegionPanel   *self,
                const gchar     *type,
@@ -641,10 +767,20 @@ add_input_row (CcRegionPanel   *self,
                const gchar     *name,
                GDesktopAppInfo *app_info)
 {
+        CcRegionPanelPrivate *priv = self->priv;
         GtkWidget *row;
         GtkWidget *label;
         GtkWidget *image;
 
+        if (priv->login) {
+                GList *l;
+                l = gtk_container_get_children (GTK_CONTAINER (priv->input_list));
+                if (l && l->next == NULL &&
+                    g_strcmp0 (g_object_get_data (G_OBJECT (l->data), "type"), "none") == 0)
+                        gtk_container_remove (GTK_CONTAINER (priv->input_list), GTK_WIDGET (l->data));
+                g_list_free (l);
+        }
+
         row = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
         label = gtk_label_new (name);
         gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -674,6 +810,9 @@ add_input_row (CcRegionPanel   *self,
                 g_object_set_data_full (G_OBJECT (row), "app-info", g_object_ref (app_info), g_object_unref);
         }
 
+        priv->n_input_rows += 1;
+        adjust_input_list_scrolling (self);
+
         return row;
 }
 
@@ -745,6 +884,9 @@ clear_input_sources (CcRegionPanel *self)
                 gtk_container_remove (GTK_CONTAINER (priv->input_list), GTK_WIDGET (l->data));
         }
         g_list_free (list);
+
+        priv->n_input_rows = 0;
+        adjust_input_list_scrolling (self);
 }
 
 static void
@@ -1052,6 +1194,9 @@ do_remove_selected_input (CcRegionPanel *self)
         gtk_container_remove (GTK_CONTAINER (priv->input_list), selected);
         egg_list_box_select_child (EGG_LIST_BOX (priv->input_list), sibling);
 
+        priv->n_input_rows -= 1;
+        adjust_input_list_scrolling (self);
+
         update_buttons (self);
         update_input (self);
 }
@@ -1217,6 +1362,7 @@ setup_input_section (CcRegionPanel *self)
         priv->remove_input = WID ("input_source_remove");
         priv->show_config = WID ("input_source_config");
         priv->show_layout = WID ("input_source_layout");
+        priv->input_scrolledwindow = WID ("input_scrolledwindow");
 
         g_signal_connect_swapped (priv->options_button, "clicked",
                                   G_CALLBACK (show_input_options), self);
@@ -1342,7 +1488,7 @@ add_input_sources_from_localed (CcRegionPanel *self)
                 g_free (id);
         }
         if (n == 0) {
-                add_input_row (self, "none", "none", C_("Input source", "None"), NULL);
+                add_input_row (self, "none", "none", _("No input source selected"), NULL);
         }
 
         g_strfreev (variants);
@@ -1350,15 +1496,14 @@ add_input_sources_from_localed (CcRegionPanel *self)
 }
 
 static void
-set_localed_locale (CcRegionPanel *self,
-                    const gchar   *language)
+set_localed_locale (CcRegionPanel *self)
 {
 	CcRegionPanelPrivate *priv = self->priv;
         GVariantBuilder *b;
         gchar *s;
 
         b = g_variant_builder_new (G_VARIANT_TYPE ("as"));
-        s = g_strconcat ("LANG=", language, NULL);
+        s = g_strconcat ("LANG=", priv->system_language, NULL);
         g_variant_builder_add (b, "s", s);
         g_free (s);
 
@@ -1434,17 +1579,22 @@ localed_proxy_ready (GObject      *source,
                      gpointer      data)
 {
         CcRegionPanel *self = data;
-	CcRegionPanelPrivate *priv = self->priv;
+        CcRegionPanelPrivate *priv;
+        GDBusProxy *proxy;
         GError *error = NULL;
 
-        priv->localed = g_dbus_proxy_new_finish (res, &error);
+        proxy = g_dbus_proxy_new_finish (res, &error);
 
-        if (!priv->localed) {
-                g_warning ("Failed to contact localed: %s\n", error->message);
+        if (!proxy) {
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Failed to contact localed: %s\n", error->message);
                 g_error_free (error);
                 return;
         }
 
+        priv = self->priv;
+        priv->localed = proxy;
+
         gtk_widget_set_sensitive (priv->login_button, TRUE);
 
         g_signal_connect (priv->localed, "g-properties-changed",
@@ -1493,7 +1643,7 @@ setup_login_button (CcRegionPanel *self)
                           "org.freedesktop.locale1",
                           "/org/freedesktop/locale1",
                           "org.freedesktop.locale1",
-                          NULL,
+                          priv->cancellable,
                           (GAsyncReadyCallback) localed_proxy_ready,
                           self);
         g_object_unref (bus);
@@ -1511,6 +1661,27 @@ setup_login_button (CcRegionPanel *self)
                                   G_CALLBACK (login_changed), self);
 }
 
+static void
+session_proxy_ready (GObject      *source,
+                     GAsyncResult *res,
+                     gpointer      data)
+{
+        CcRegionPanel *self = data;
+        GDBusProxy *proxy;
+        GError *error = NULL;
+
+        proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
+
+        if (!proxy) {
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Failed to contact gnome-session: %s\n", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        self->priv->session = proxy;
+}
+
 static void
 cc_region_panel_init (CcRegionPanel *self)
 {
@@ -1533,6 +1704,18 @@ cc_region_panel_init (CcRegionPanel *self)
 
         priv->user_manager = act_user_manager_get_default ();
 
+        priv->cancellable = g_cancellable_new ();
+
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                  G_DBUS_PROXY_FLAGS_NONE,
+                                  NULL,
+                                  "org.gnome.SessionManager",
+                                  "/org/gnome/SessionManager",
+                                  "org.gnome.SessionManager",
+                                  priv->cancellable,
+                                  session_proxy_ready,
+                                  self);
+
         setup_login_button (self);
         setup_language_section (self);
         setup_input_section (self);
