@@ -46,8 +46,6 @@ struct _UmPasswordDialog {
         GtkWidget *verify_entry;
         GtkWidget *strength_indicator;
         GtkWidget *strength_indicator_label;
-        GtkWidget *normal_hint_entry;
-        GtkWidget *normal_hint_label;
         GtkWidget *show_password_button;
         GtkWidget *ok_button;
 
@@ -68,6 +66,31 @@ typedef enum {
         UM_PASSWORD_DIALOG_MODE_UNLOCK_ACCOUNT
 } UmPasswordDialogMode;
 
+static int
+update_password_strength (UmPasswordDialog *um)
+{
+        const gchar *password;
+        const gchar *old_password;
+        const gchar *username;
+        gint strength_level;
+        const gchar *hint;
+        const gchar *long_hint;
+
+        password = gtk_entry_get_text (GTK_ENTRY (um->password_entry));
+        old_password = gtk_entry_get_text (GTK_ENTRY (um->old_password_entry));
+        username = act_user_get_user_name (um->user);
+
+        pw_strength (password, old_password, username,
+                     &hint, &long_hint, &strength_level);
+
+        gtk_level_bar_set_value (GTK_LEVEL_BAR (um->strength_indicator), strength_level);
+        gtk_label_set_label (GTK_LABEL (um->strength_indicator_label), hint);
+        gtk_widget_set_tooltip_text (um->strength_indicator, long_hint);
+        gtk_widget_set_tooltip_text (um->strength_indicator_label, long_hint);
+
+        return strength_level;
+}
+
 static void
 generate_one_password (GtkWidget        *widget,
                        UmPasswordDialog *um)
@@ -116,7 +139,6 @@ finish_password_change (UmPasswordDialog *um)
 
         gtk_entry_set_text (GTK_ENTRY (um->password_entry), " ");
         gtk_entry_set_text (GTK_ENTRY (um->verify_entry), "");
-        gtk_entry_set_text (GTK_ENTRY (um->normal_hint_entry), "");
         gtk_entry_set_text (GTK_ENTRY (um->old_password_entry), "");
 
         um_password_dialog_set_user (um, NULL);
@@ -203,7 +225,7 @@ accept_password_dialog (GtkButton        *button,
         gtk_tree_model_get (model, &iter, 1, &mode, -1);
 
         password = gtk_entry_get_text (GTK_ENTRY (um->password_entry));
-        hint = gtk_entry_get_text (GTK_ENTRY (um->normal_hint_entry));
+        hint = NULL;
 
         switch (mode) {
                 case UM_PASSWORD_DIALOG_MODE_NORMAL:
@@ -259,18 +281,27 @@ update_sensitivity (UmPasswordDialog *um)
         const gchar *old_password;
         const gchar *tooltip;
         gboolean can_change;
+        int strength_level;
 
         password = gtk_entry_get_text (GTK_ENTRY (um->password_entry));
         verify = gtk_entry_get_text (GTK_ENTRY (um->verify_entry));
         old_password = gtk_entry_get_text (GTK_ENTRY (um->old_password_entry));
 
-        if (strlen (password) < pw_min_length ()) {
+        /* Don't update the password strength if we didn't enter anything */
+        if (password && *password == '\0' &&
+            verify && *verify == '\0' &&
+            old_password && *old_password == '\0')
+                return;
+
+        strength_level = update_password_strength (um);
+
+        if (strength_level < 1) {
                 can_change = FALSE;
                 if (password[0] == '\0') {
                         tooltip = _("You need to enter a new password");
                 }
                 else {
-                        tooltip = _("The new password is too short");
+                        tooltip = _("The new password is not strong enough");
                 }
         }
         else if (strcmp (password, verify) != 0) {
@@ -312,8 +343,6 @@ action_changed (GtkComboBox      *combo,
                 gtk_entry_set_icon_sensitive (GTK_ENTRY (um->password_entry), GTK_ENTRY_ICON_SECONDARY, TRUE);
                 gtk_widget_set_sensitive (um->verify_entry, TRUE);
                 gtk_widget_set_sensitive (um->old_password_entry, TRUE);
-                gtk_widget_set_sensitive (um->normal_hint_entry, TRUE);
-                gtk_widget_set_sensitive (um->normal_hint_label, TRUE);
                 gtk_widget_set_sensitive (um->strength_indicator_label, TRUE);
                 gtk_widget_set_sensitive (um->show_password_button, TRUE);
 
@@ -324,8 +353,6 @@ action_changed (GtkComboBox      *combo,
                 gtk_entry_set_icon_sensitive (GTK_ENTRY (um->password_entry), GTK_ENTRY_ICON_SECONDARY, FALSE);
                 gtk_widget_set_sensitive (um->verify_entry, FALSE);
                 gtk_widget_set_sensitive (um->old_password_entry, FALSE);
-                gtk_widget_set_sensitive (um->normal_hint_entry, FALSE);
-                gtk_widget_set_sensitive (um->normal_hint_label, FALSE);
                 gtk_widget_set_sensitive (um->strength_indicator_label, FALSE);
                 gtk_widget_set_sensitive (um->show_password_button, FALSE);
                 gtk_widget_set_sensitive (um->ok_button, TRUE);
@@ -343,29 +370,6 @@ show_password_toggled (GtkToggleButton  *button,
         gtk_entry_set_visibility (GTK_ENTRY (um->verify_entry), active);
 }
 
-static void
-update_password_strength (UmPasswordDialog *um)
-{
-        const gchar *password;
-        const gchar *old_password;
-        const gchar *username;
-        gint strength_level;
-        const gchar *hint;
-        const gchar *long_hint;
-
-        password = gtk_entry_get_text (GTK_ENTRY (um->password_entry));
-        old_password = gtk_entry_get_text (GTK_ENTRY (um->old_password_entry));
-        username = act_user_get_user_name (um->user);
-
-        pw_strength (password, old_password, username,
-                     &hint, &long_hint, &strength_level);
-
-        gtk_level_bar_set_value (GTK_LEVEL_BAR (um->strength_indicator), strength_level);
-        gtk_label_set_label (GTK_LABEL (um->strength_indicator_label), hint);
-        gtk_widget_set_tooltip_text (um->strength_indicator, long_hint);
-        gtk_widget_set_tooltip_text (um->strength_indicator_label, long_hint);
-}
-
 static void
 update_password_match (UmPasswordDialog *um)
 {
@@ -424,14 +428,6 @@ verify_entry_focus_out (GtkWidget        *entry,
         return FALSE;
 }
 
-static void
-entry_size_changed (GtkWidget     *entry,
-                    GtkAllocation *allocation,
-                    GtkWidget     *label)
-{
-        gtk_widget_set_size_request (label, allocation->width, -1);
-}
-
 static void
 auth_cb (PasswdHandler    *handler,
          GError           *error,
@@ -608,16 +604,6 @@ um_password_dialog_new (void)
         widget = (GtkWidget *) gtk_builder_get_object (builder, "strength-indicator-label");
         gtk_label_set_width_chars (GTK_LABEL (widget), len);
 
-        um->normal_hint_entry = (GtkWidget *) gtk_builder_get_object (builder, "normal-hint-entry");
-
-        /* Label size hack.
-         * This only sort-of works because the dialog is non-resizable.
-         */
-        widget = (GtkWidget *)gtk_builder_get_object (builder, "password-normal-hint-description-label");
-        g_signal_connect (um->normal_hint_entry, "size-allocate",
-                          G_CALLBACK (entry_size_changed), widget);
-        um->normal_hint_label = widget;
-
         um->strength_indicator = (GtkWidget *) gtk_builder_get_object (builder, "strength-indicator");
 
         um->strength_indicator_label = (GtkWidget *) gtk_builder_get_object (builder, "strength-indicator-label");
@@ -687,7 +673,6 @@ um_password_dialog_set_user (UmPasswordDialog *um,
 
                 gtk_entry_set_text (GTK_ENTRY (um->password_entry), "");
                 gtk_entry_set_text (GTK_ENTRY (um->verify_entry), "");
-                gtk_entry_set_text (GTK_ENTRY (um->normal_hint_entry), "");
                 gtk_entry_set_text (GTK_ENTRY (um->old_password_entry), "");
                 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (um->show_password_button), FALSE);
                 if (act_user_get_uid (um->user) == getuid () &&
